unit GardePositionFiche;


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,iniFiles;

type
  TGardePositionFiche = class(TComponent)
  private
    FSaveWindowState : Boolean;
    FReadWindowState : Boolean;
    FEnabled         : Boolean;
    FFileName        : TFileName;
    PremiereFois     : BooLean;//Permet de ne dclencher la lecture qu'une fois au dbnut du programme
    Function DetermineNomFic : String;
  protected
    FormAOwner:TForm;
    FormOldShow: TNotifyEvent;
    FormOldClose:TCloseEvent;
    procedure LaFormClose(Sender: TObject; var Action: TCloseAction);
    procedure LaFormShow(sender: TObject);
    procedure ExecuteDepart(Sender:TForm);
    procedure ExecuteFin(Sender:TForm);
  public
     Constructor Create(AOwner:TComponent); override;
  published
     property SaveWindowState:Boolean read FSaveWindowState write FSaveWindowState;
     property ReadWindowState:Boolean read FReadWindowState write FReadWindowState;
     property Enabled  : Boolean read FEnabled write FEnabled;
     property FileName : TFileName read FFileName write FFileName;
  end;

procedure Register;

implementation



Constructor TGardePositionFiche.Create(AOwner:TComponent);
begin
  Inherited create(AOwner);
  if not (csDesigning in ComponentState) then //si on est pas en mode conception
  begin
    FormAOwner:=TForm(AOwner);// la forme propritaire de notre composant
    FormOldShow:=FormAOwner.OnShow; // sauvegarde de l'vnement OnShow dans FormOldShow
    FormOldClose:=FormAOwner.OnClose; //idem pour OnClose
    FormAOwner.OnShow:=LaFormShow; // on redirige l'vnement OnShow vers LaFormShow
    FormAOwner.OnClose:=LaFormClose; //idem pour OnClose
    PremiereFois:=True;
  end
  else
  begin // si on est en mode conception
    FSaveWindowState    :=True;
    FReadWindowState    :=True;
    FEnabled            :=True;
    FFileName           :='';
  end;
end;

procedure TGardePositionFiche.LaFormShow(sender: TObject);
begin
  if assigned(FormOldShow) then FormOldShow(sender);
  if FEnabled and PremiereFois then
  begin
    ExecuteDepart(FormAOwner);
    PremiereFois:=False;// permet de n'y passer qu'une seule fois
  end;
end;

procedure TGardePositionFiche.LaFormClose(Sender: TObject; var Action: TCloseAction);
begin
  if assigned(FormOldClose)then FormOldClose(Sender,Action);
  if FEnabled then ExecuteFin(FormAOwner);
end;


procedure TGardePositionFiche.ExecuteDepart(Sender:TForm);
var FIni:TiniFile;
    NomFic : String;
    WindowPlacement : TWindowPlacement;
begin
  NomFic:=DetermineNomFic;
  FIni:=TiniFile.Create(NomFic);
  try
    if Fini.ValueExists(Sender.Name,'Height') and Fini.ValueExists(Sender.name,'Top')   and
       Fini.ValueExists(Sender.name,'Left')   and Fini.ValueExists(Sender.name,'Width') then
    begin
      if FReadWindowState and Fini.ValueExists(Sender.name,'WindowState') then Sender.WindowState := TWindowState (FIni.ReadInteger(Sender.name,'WindowState',-1));
      // rcupration du WindowPlacement actuel car on ne va en modifier que certaines valeurs
      WindowPlacement.length:=SizeOf(WindowPlacement);
      GetWindowPlacement(Sender.handle,@WindowPlacement); // rcupration du WindowState avant le WindowPlacement de faon  ce qu'il contienne le bon WindowState.
      // rcupration de ce qui est dans le fichier ini
      WindowPlacement.rcNormalPosition.Top    := FIni.ReadInteger(Sender.name,'Top',-1);
      WindowPlacement.rcNormalPosition.Left   := FIni.ReadInteger(Sender.name,'Left',-1);
      WindowPlacement.rcNormalPosition.Right  := WindowPlacement.rcNormalPosition.Left+FIni.ReadInteger(Sender.name,'Width',-1);
      WindowPlacement.rcNormalPosition.Bottom := WindowPlacement.rcNormalPosition.Top +FIni.ReadInteger(Sender.name,'Height',-1);
      // modification des proprits de la fentre en fonction du WindowPlacement
      SetWindowPlacement(Sender.handle,@WindowPlacement);
    end;// fin si
  finally
    Fini.Free;
  end;
end;

procedure TGardePositionFiche.ExecuteFin(Sender: TForm);
var FIni            :TiniFile;
    NomFic          :string;
    WindowPlacement : TWindowPlacement;
begin
  NomFic:=DetermineNomFic;
  FIni:=TiniFile.Create(NomFic);
  try
    WindowPlacement.length:=SizeOf(WindowPlacement); //  faire avant l'appel  GetWindowPlacement
    GetWindowPlacement(Sender.handle,@WindowPlacement);// rcupre entre autres l'emplacement de la fentre
    try
      if FSaveWindowState then FIni.WriteInteger(Sender.name,'WindowState',ord(Sender.WindowState));
      FIni.WriteInteger(Sender.name, 'Height' ,WindowPlacement.rcNormalPosition.Bottom-WindowPlacement.rcNormalPosition.Top);
      FIni.WriteInteger(Sender.name, 'Top'    ,WindowPlacement.rcNormalPosition.Top);
      FIni.WriteInteger(Sender.name, 'Left'   ,WindowPlacement.rcNormalPosition.Left);
      FIni.WriteInteger(Sender.name, 'Width'  ,WindowPlacement.rcNormalPosition.Right-WindowPlacement.rcNormalPosition.Left);
    except
    end;
  finally
    Fini.Free;
  end;
end;


Function TGardePositionFiche.DetermineNomFic : String;
Var NomFic: string;
begin
  NomFic:='';
  if FFileName<> '' then
  begin // FFileName n'est pas vide
    if (ExtractFilePath(FFileName)<>'') or (FFileName[1]='\') then NomFic:=FFileName
    else NomFic:= ExtractFilePath(application.ExeName)+FFileName;
  end;
  if ExtractFileName(NomFic)='' then
  Begin
    NomFic:=ExtractFileName(application.ExeName);
    NomFic:=copy(NomFic,1,Length(NomFic)-4)+'.ini';
  end;
  if ExtractFileExt(NomFic)='' then NomFic:=NomFic+'.ini';
  Result:=NomFic;
end;

procedure Register;
begin
  RegisterComponents('Perso', [TGardePositionFiche]);
end;




end.
